#ifndef _CAMD_COMMON_H_
#define _CAMD_COMMON_H_

#define ZERO(x) (memset(&(x), 0, sizeof(x)))
#define CAMD_DIR "/etc/camd"

/* required for exit() */
#include <stdlib.h>
/* access errno from assert */
#include <errno.h>

/* for strerr() */
#include <string.h>

#ifdef USE_CAMD_SPEW
#include "spew.h"

#define __ASSERT(x, file, line, function) do { \
	if (!(x)) \
	{ \
		SPEW(0, "[%s:%d] in %s() fatal failure detected, expr='%s'\n" \
				"libc state {errno=%d,strerr='%s'}\n", \
				file, line, function, #x, errno, strerror(errno)); \
		exit(-1); \
	} \
} while (0)

#else /* USE_CAMD_SPEW */

/* for fprintf */
#include <stdio.h>

#define __ASSERT(x, file, line, function) do { \
	if (!(x)) \
	{ \
		fprintf(stderr, "[%s:%d] in %s() crucial, expr='%s'\n" \
				"libc state {errno=%d,strerr='%s'}\n", \
				file, line, function, #x, errno, strerror(errno)); \
		*(int *)0 = 0; \
	} \
} while (0)

#endif /* USE_CAMD_SPEW */

#define ASSERT(x) __ASSERT(x, __FILE__, __LINE__, __FUNCTION__)

#ifdef DEBUG

/* called on in DEBUG mode */
#define DEBUG_ASSERT(x) __ASSERT(x, __FILE__, __LINE__, __FUNCTION__)
#define DEBUG_CALL(x)  (void)(x)

void module_register(const char *name, int *magic_ptr);
void module_magic_alert(int expected_magic, int found_magic);

#define MODULE_DECLARE_MAGIC_STATIC static int module_magic
#define MODULE_DECLARE_MAGIC_MEMBER int magic
#define MODULE_REGISTER(x, type) \
	do { \
		module_register(#type, &module_magic); \
		x->magic = module_magic; \
	} while (0)

#define MODULE_VERIFY(x) \
	do { \
		if (x->magic != module_magic) \
		{ \
			module_magic_alert(module_magic, x->magic); \
			DEBUG_ASSERT(0); \
		} \
	} while (0)

#else /* DEBUG */

/* compiled out */
#define DEBUG_ASSERT(x) do {} while (0)
#define DEBUG_CALL(x) do {} while (0)

#define MODULE_DECLARE_MAGIC_STATIC
#define MODULE_DECLARE_MAGIC_MEMBER
#define MODULE_REGISTER(x, type)
#define MODULE_VERIFY(x)

#endif /* DEBUG */

typedef enum {
	STATE_OFF,
	STATE_ON,
	STATE_AUTO,
	STATE_NONE,
	STATE_LAST,
} camd_state_t;

const char *state2str(camd_state_t state);

#endif /* _CAMD_COMMON_H_ */
